@@ -56,9 +56,11 @@ struct
     if not withtxn then 
       failwith "Running sks_db without transactions is no longer supported."
 
+  let websocks =
+    List.map ~f:Eventloop.create_sock
+      ((if !Settings.use_port_80 then make_addr_list http_address 80 else [])
+       @ make_addr_list http_address http_port)
 
-  let addr = inet_addr_of_string http_address
-  let websock = Eventloop.create_sock (ADDR_INET (addr,http_port))
   let () = 
     if Sys.file_exists db_command_name 
     then Unix.unlink db_command_name
@@ -110,9 +112,9 @@ struct
   let get_stats () = 
     let today = Stats.round_up_to_day (Unix.gettimeofday ()) in
     let log = 
-      let maxsize = 20000 in
-      let last_week = today -. (7. *. 24. *. 60. *. 60.) in
-      Keydb.reverse_logquery ~maxsize last_week
+      let maxsize = 180000 in
+      let last_month = today -. (180. *. 24. *. 60. *. 60.) in
+      Keydb.reverse_logquery ~maxsize last_month
     in
     let size = Keydb.get_num_keys () in
     (log,size)
@@ -136,11 +138,17 @@ struct
 
       | 8 -> (* 64-bit keyid *) 
 	  List.filter keys
-	  ~f:(fun key -> (Fingerprint.from_key key).Fingerprint.keyid = keyid )
+	  ~f:(fun key -> keyid = (Fingerprint.from_key key).Fingerprint.keyid ||
+	  (** Return keys i& subkeys with matching long keyID *)
+	     let (mainkeyid,subkeyids) = Fingerprint.keyids_from_key ~short:false key in
+	     List.exists (fun x -> x = keyid) subkeyids)
 
       | 20 -> (* 160-bit v. 4 fingerprint *)
 	  List.filter keys
-	  ~f:(fun key -> keyid = (Fingerprint.from_key key).Fingerprint.fp )
+	  ~f:(fun key -> keyid = (Fingerprint.from_key key).Fingerprint.fp ||
+	  (** Return keys & subkeys with matching fingerprints *)
+              let (mainkeyfp,subkeyfps) = Fingerprint.fps_from_key key in
+              List.exists (fun x -> x = keyid) subkeyfps)
 
       | 16 -> (* 128-bit v3 fingerprint.  Not supported *)
 	  failwith "128-bit v3 fingerprints not implemented"
@@ -203,20 +211,35 @@ struct
 
 
   (******************************************************************)
+  let truncate count keys =
+    let rec trunc_c result orig num =
+      match orig with
+        | [] -> result
+	| h::tail ->
+            if (num = 0)
+	    then result
+	    else (trunc_c (result @ [h]) tail (num-1))
+    in
+    if count > 0
+    then trunc_c [] keys count 
+    else keys
 
   let handle_get_request request =
     match request.kind with
       | Stats -> 
 	  plerror 4 "/pks/lookup: DB Stats request";
-	  ("text/html; charset=UTF-8", !last_stat_page)
+	  ("text/html; charset=UTF-8", -1, !last_stat_page)
       | Get -> 
 	  plerror 4 "/pks/lookup: Get request (%s)"
 	    (String.concat " " request.search);
 	  let keys = lookup_keys request.search in
 	  let keys = clean_keys request keys in
+	  let count = List.length keys in
+	  let keys = truncate request.limit keys in
 	  let keystr = Key.to_string_multiple keys in
 	  let aakeys = Armor.encode_pubkey_string keystr in
 	  ("text/html; charset=UTF-8",
+	   count,
 	   HtmlTemplates.page  
 	     ~title:(sprintf "Public Key Server -- Get ``%s ''" 
 		       (String.concat ~sep:" " request.search))
@@ -241,6 +264,7 @@ struct
 	  let keystr = Key.to_string key in
 	  let aakeys = Armor.encode_pubkey_string keystr in
 	  ("text/html; charset=UTF-8",
+	   1,
 	   HtmlTemplates.page  
 	     ~title:(sprintf "Public Key Server -- Get ``%s ''" hash_str)
 	     ~body:(sprintf "\r\n<pre>\r\n%s\r\n</pre>\r\n" aakeys)
@@ -251,10 +275,13 @@ struct
 	  plerror 4 "/pks/lookup: Index request: (%s)" 
 	    (String.concat " " request.search);
 	  let keys = lookup_keys request.search in
+	  let count = List.length keys in
+	  let keys = truncate request.limit keys in
 	  let hashes = List.map ~f:KeyHash.hash keys in
 	  let keys = clean_keys request keys in
 	  if request.machine_readable then 
 	    ("text/plain",
+	     count,
 	     MRindex.keys_to_index keys)
 	  else 
 	    begin
@@ -273,6 +300,7 @@ struct
 			    (Index.keyinfo_header request :: output)
 		in
 		("text/html; charset=UTF-8",
+		 count,
 		 HtmlTemplates.page ~body:pre
 		   ~title:(sprintf "Search results for '%s'" 
 			     (String.concat ~sep:" " request.search))
@@ -348,7 +376,7 @@ struct
 
   let is_safe char = 
     (char >= 'A' && char <= 'Z') || (char >= 'a' && char <= 'z') || 
-    (char >= '0' && char <= '9') || (char = '.')
+    (char >= '0' && char <= '9') || (char = '.') || (char = '-')
     
 
   let verify_web_fname fname = 
@@ -366,11 +394,21 @@ struct
     else raise (Wserver.Misc_error "Malformed requst")
 
   let supported_extensions = 
-    [ ".jpg", "image/jpeg";
-      ".gif", "image/gif";
-      ".ico", "image/x-icon";
-      ".png", "image/png";
-      ".txt", "text/plain"; 
+    [ ".jpg",   "image/jpeg";
+      ".jpeg",  "image/jpeg";
+      ".gif",   "image/gif";
+      ".ico",   "image/x-icon";
+      ".png",   "image/png";
+      ".htm",   "text/html";
+      ".html",  "text/html";
+      ".shtml", "text/html";
+      ".txt",   "text/plain"; 
+      ".css",   "text/css";
+      ".xhtml", "application/xhtml+xml";
+      ".xhtm",  "application/xhtml+xml";
+      ".xml",   "application/xhtml+xml";
+      ".es",    "application/ecmascript";
+      ".js",    "application/javascript";
     ]
 
   (** Handler for HTTP requests *)
@@ -382,9 +420,9 @@ struct
 	  let (base,oplist) = string_to_oplist request in
 	  if base = "/pks/lookup" then (
 	    let request = request_of_oplist oplist in
-	    let (mimetype,body) = handle_get_request request in
+	    let (mimetype,count,body) = handle_get_request request in
 	    cout#write_string body;
-	    mimetype
+	    (mimetype, count)
 	  ) else (
 	    if (base = "/index.html" || base = "/index.htm" 
 		|| base = "/" || base = "")
@@ -392,7 +430,7 @@ struct
 	      let fname = convert_web_fname "index.html" in 
 	      let text = read_file fname in
 	      cout#write_string text;
-	      "text/html; charset=UTF-8"
+	      ("text/html; charset=UTF-8", -1)
 	    else 
 	      (try 
 		 let extension = get_extension base in
@@ -406,7 +444,7 @@ struct
 		 let base = base </> (1,0) in
 		 let data = read_file ~binary:true (convert_web_fname base) in
 		 cout#write_string data;
-		 mimetype
+		 (mimetype, -1)
 	       with
 		   Not_found -> raise (Wserver.Page_not_found base)
 	      )
@@ -452,10 +490,10 @@ struct
 		  cout#write_string 
 		    ("Key block added to key server database.\n  " ^
 		     "New public keys added: <br>");
-		  cout#write_string (sprintf "%d keys added succesfully.<br>" !ctr)
+		  cout#write_string (sprintf "%d key(s) added successfully.<br>" !ctr)
 		);
 		cout#write_string "</html></body>";
-		"text/html; charset=UTF-8"
+		("text/html; charset=UTF-8", List.length keys)
 	    | "/pks/hashquery" ->
 		plerror 4 "Handling /pks/hashquery"; 
 		let sin = new Channel.string_in_channel body 0 in
@@ -466,12 +504,13 @@ struct
 		perror "%d keys found" (List.length keystrings);
 		CMarshal.marshal_list ~f:CMarshal.marshal_string cout 
 		  keystrings;
-		"pgp/keys" (* This is a bogus content-type *)
+		("pgp/keys" (* This is a bogus content-type *),
+		 List.length keystrings)
 	    | _ ->
 		cout#write_string (HtmlTemplates.page 
 				     ~title:"Unexpected POST request" 
 				     ~body:"");
-		"text/html; charset=UTF-8"
+		("text/html; charset=UTF-8", -1)
 
 
   (** Prepare handler for use with eventloop by transforming system
@@ -618,6 +657,20 @@ struct
 
   (***********************************************************************)
 
+  let sync_db_on_sig () =
+    sync ();
+    checkpoint ()
+
+  let () = Sys.set_signal Sys.sigusr1
+	  (Sys.Signal_handle (fun _ -> sync_db_on_sig ()))
+
+  let () = Sys.set_signal Sys.sigusr2
+      (Sys.Signal_handle (fun _ ->
+	Eventloop.add_events Eventloop.heap
+	  [Eventloop.Event(0.0, Eventloop.Callback calculate_stats_page)]))
+
+  (***********************************************************************)
+
   let run () = 
     Keydb.open_dbs settings;
     if !Settings.initial_stat then ignore (calculate_stats_page ());
@@ -631,44 +684,36 @@ struct
 	 then (Ehandlers.repeat_forever_simple checkpoint_interval checkpoint)
 	 else (Ehandlers.repeat_forever_simple sync_interval sync)) 
 	@
-	Ehandlers.repeat_forever_simple !Settings.membership_reload_time
-	 Membership.reset_membership_time
+	  Ehandlers.repeat_forever_simple !Settings.membership_reload_time
+	  Membership.reset_membership_time
 	@
-	(Ehandlers.repeat_forever 10. 
-	   (Eventloop.make_tc ~cb:transmit_keys ~timeout:0
-	      ~name:"mail transmit keys" )
-	)
+	  (if !Settings.send_mailsyncs then
+	     (Ehandlers.repeat_forever 10. 
+	        (Eventloop.make_tc ~cb:transmit_keys ~timeout:0
+	           ~name:"mail transmit keys" )
+	     )
+	   else [])
 	@
-	(if !Settings.send_mailsyncs then
-	   (Ehandlers.repeat_forever 10. 
-	      (Eventloop.make_tc ~name:"mailsync" ~timeout:0
-		 ~cb:(Mailsync.load_mailed_keys 
-			~addkey:(Keydb.add_key_merge ~newkey:false)))
-	   )
-	 else [])
+	  (Ehandlers.repeat_forever 10. 
+	     (Eventloop.make_tc ~name:"mailsync" ~timeout:0
+		~cb:(Mailsync.load_mailed_keys 
+		       ~addkey:(Keydb.add_key_merge ~newkey:false)))
+	  )
 	@
-	(Ehandlers.repeat_at_hour !Settings.stat_calc_hour
-	   calculate_stats_page)
+	  (Ehandlers.repeat_at_hour !Settings.stat_calc_hour
+	     calculate_stats_page)
       )
 
       (
-	(websock, Eventloop.make_th ~name:"webserver" 
-	  ~timeout:!Settings.wserver_timeout
-	  ~cb:(Wserver.accept_connection webhandler ~recover_timeout:1))
-	::
 	 (comsock, Eventloop.make_th ~name:"command handler" 
 	    ~timeout:!Settings.command_timeout
 	    ~cb:(eventify_handler command_handler))
 	::
-	 (if !Settings.use_port_80 then
-	    let sock = Eventloop.create_sock (ADDR_INET (addr,80)) in
-	    (sock,Eventloop.make_th ~name:"webserver80" 
-	       ~timeout:!Settings.wserver_timeout
-	       ~cb:(Wserver.accept_connection webhandler ~recover_timeout:1)
-	    )::[]
-	  else
-	    []
-	 )
+	 (List.map websocks
+	    ~f:(fun sock ->
+	          (sock, Eventloop.make_th ~name:"webserver" 
+		     ~timeout:!Settings.wserver_timeout
+		     ~cb:(Wserver.accept_connection webhandler ~recover_timeout:1))))
       )
 
 
