@@ -539,8 +539,11 @@ struct
 	let lengths = List.map ~f:Cursor.count cursors in
 	if MList.min lengths > max_internal_matches
 	then raise (Invalid_argument "Insufficiently specific words");
-	let cj = Cursor.join dbs.key cursors [] in
-	let keystrings = jcursor_get_all ~max cj in
+	let keystrings =
+	  let cj = Cursor.join dbs.key cursors [] in
+	  protect ~f:(fun () -> jcursor_get_all ~max cj)
+	    ~finally:(fun () -> Cursor.close cj)
+	in
 	if List.length keystrings >= max then
 	  raise (Invalid_argument "Too many responses")
 	else
@@ -1261,7 +1264,6 @@ struct
 
   (** replace [key1] with [key2] in the database *)
   let swap_keys key1 key2 = 
-    let dbs = get_dbs () in
     let txn = txn_begin () in
     try
       delete_key_txn ?txn key1;
@@ -1281,7 +1283,6 @@ struct
   (**********************************************************)
 
   let delete_key ?hash key = 
-    let dbs = get_dbs () in
     let txn = txn_begin () in
     try
       delete_key_txn ?txn key;
